# Translation for Developers

When writing new application code, developers should organize their code in a way which is amenable to internationalization, so that it may be localized to various languages and regions of the world.

If you are an extension developer, there is additional documentation in the [Extension translation](/translation/extensions.md) page.

## PHP

The strings hard-coded into PHP should be wrapped in `ts()` function calls. Here are a few examples:

```php
$string = ts('Hello, World!');

$group = array('' => ts('- any group -')) + $this->_group;
```

You can also use placeholders for variables:

```php
$string = ts("A new '%1' has been created.", array(1 => $contactType));
```

Note that variables should themselves be translated by your code before passing in, if appropriate.

If the string might be singular or plural, use the following syntax:

```php
$string = ts('%count item created', array('count' => $total, 'plural' => '%count items created'));
```

A few examples to avoid:

```php
// Bad: Avoid escaped quotes: this is harder to read:
$string = ts('A new \'%1\' has been created.', array(1 => $contactType));

// Good:
$string = ts("A new '%1' has been created.", array(1 => $contactType));

// Bad: multi-line strings:
// Even if your code editor may not like it, this should be on a single line
// since a change in indentation might change where the line breaks are, which
// would then require re-translating the string.
$string = ts("Lorem ipsum dolor sit amet, consectetur adipiscing elit. Proin 
  elementum, ex in pretium tincidunt, felis lorem facilisis lacus, vel 
  iaculis ex orci vitae risus. Maecenas in sapien ut velit scelerisque 
  interdum.");

// Bad:
$string = ts('%1 item(s) created', array(1 => $count));

// Good:
$string = ts('%count item created', array('count' => $total, 'plural' => '%count items created'));
```

Another common error is to use `ts()` to aggregate strings or as a "clever" way of writing shorter code:

```php
// Bad: incorrect aggregation
// This will be extremely confusing to translations
// and might give some really bad results in some languages.
$operation = empty($params['id']) ? ts('New') : ts('Edit'));
$string = ts("%1 %2", array(1 => $operation, 2 => $contactType));

// Less bad:
// Note that this still makes it difficult to use the correct gender.
if (empty($params['id'])) {
  $string = ts("New %1", array(1 => $contactType));
}
else {
  $string = ts("Edit %1", array(1 => $contactType));
}
```

## Javascript

When translating strings in an extension, ts scope needs to be declared. The `CRM.ts` function takes scope as an argument and returns a function that always applies that scope to ts calls:

```js
// This closure gets a local copy of jQuery, Lo-dash, and ts
(function($, _, ts) {
  CRM.alert(ts('Your foo has been barred.'));
})(CRM.$, CRM._, CRM.ts('foo.bar.myextension'));
```

!!! note
    `CRM.ts` is not the same as the global `ts` function. `CRM.ts` is a function that returns a function (javascript is wacky like that). Since your closure gives the local `ts` the same name as the global `ts`, it will be used instead.

!!! important
    Your local version of `ts` could be named anything, but strings in your javascript file cannot be accurately parsed unless you name it `ts`.

## Smarty templates

* The strings hard-coded into templates should be wrapped in `{ts}...{/ts}` tags. For example:

    ```smarty
    {ts}Full or partial name (last name, or first name, or organization name).{/ts}
    ```

* If you need to pass a variable to the localizable string, you should use the following pattern:

    ```smarty
    <div class="status">
      {ts 1=$delName}Are you sure you want to delete <b>%1</b> Tag?{/ts}
    </div>
    ```

* When possible, avoid HTML formatting and newlines inside `{ts}...{/ts}` tags. 

    * Good

        ```smarty
        <p>{ts}Hello, world!{/ts}</p>
        ```
    
    * Bad
    
        ```
        {ts}<p>Hello, world!</p>{/ts}
        ```

## Rationale for using Gettext

In most projects, strings are typically translated by either:

* using Gettext (which is what CiviCRM does),
* using arrays of key/string dictionaries,
* using database lookups of strings (which is what Drupal does).

In order to be support Joomla!, WordPress, Backdrop and eventually other content management systems. Gettext is the standard way to translate strings in PHP, used by most projects.

## Other guides/references

Here are the guides to other popular projects:

* Drupal: <https://www.drupal.org/node/322729>
* Joomla!: <https://docs.joomla.org/Specification_of_language_files>
* WordPress: <https://codex.wordpress.org/I18n_for_WordPress_Developers>
